# Selectricity: Voting Machinery for the Masses
# Copyright (C) 2007, 2008 Benjamin Mako Hill <mako@atdot.cc>
# Copyright (C) 2007 Massachusetts Institute of Technology
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see
# <http://www.gnu.org/licenses/>.

class VoterController < ApplicationController
  helper :sparklines
  layout 'main'
  require_dependency "voter"
  require_dependency "vote"
  require_dependency "election"

  def index
    if params[:election_id]
      @election = Election.find(params[:election_id])
      unless @election.authenticated?
        @voter = Voter.find(:all,
          :conditions => ["session_id = ? and election_id = ?",
          session.session_id, @election.id])[0]
      
        @voter = Voter.new unless @voter

        @voter.election = @election
        @voter.session_id = session.session_id
        @password = "open." + @election.id.to_s
      end
    elsif params[:urlpassword]
      password = params[:urlpassword]

      if @voter = FullVoter.find(:all,
        :conditions => [ "password = ?", password ] )[0]
        @election = @voter.election
        @password = @voter.password
      end
    end

    if @voter and @election
      # initialize things if the vote is blank
      if @voter.vote.nil?
        @voter.vote = Vote.new 
        @voter.save
      end
      
      @voter.vote.set_defaults! if @voter.vote.rankings.empty?

      # if the election is now finished 
      if @election.enddate < Time.now
        redirect_to :action => :results, :id => @password
      else
        @sidebar_content = render_to_string(:partial => 'vote_sidebar')
        if @election.embeddable? and params[:embed] == "true"
          #look for custom theme, and assign to instance variabels for widget use
          if @election.embed_custom_string
            @top_bar = SkinPicture.find(:first,
            :conditions => ["filename = ?", @election.embed_custom_string + "top_bar.png"])
            @default_image = SkinPicture.find(:first,
            :conditions => ["filename = ?", @election.embed_custom_string + "default_image.png"])
            @bg1 = SkinPicture.find(:first,
            :conditions => ["filename = ?", @election.embed_custom_string + "bg1.png"])
            @bg2 = SkinPicture.find(:first,
            :conditions => ["filename = ?", @election.embed_custom_string + "bg2.png"])
            @bottom_bar = SkinPicture.find(:first,
            :conditions => ["filename = ?", @election.embed_custom_string + "bottom_bar.png"])
          end
          render :template => 'embed/full_vote', :layout => 'embed'
        else
          render :action => 'full_vote'
        end
      end
    end
  end

  def login
    if params[:vote] and params[:vote][:password]
      redirect_to votepassword_url( :action => 'index', :urlpassword => params[:vote][:password])
    else
      redirect_to :action => 'index'
    end
  end
  
  def pref_tables
    if authenticate
      @election = @voter.election
      @results = @election.results
      @candidates = {}
      @election.candidates.each {|c| @candidates[c.id] = c}
      @names = @election.names_by_id
      render :template => 'common/pref_tables', :layout => 'basic'
    else
      redirect_to :action => 'index'
    end
  end

  def details
    if authenticate
      @election = @voter.election
      @votes = @election.votes.select {|v| v.confirmed? }.shuffle
      @voters = @votes.collect {|v| v.voter}.shuffle
      render :action => 'details'
    else
      redirect_to :action => 'index'
    end
  end

  def review
    if authenticate
      @voter.vote.time = Time.now
      @voter.vote.save
      @voter.reload
    else
      redirect_to :action => 'index'
    end
  end

  def confirm
    if authenticate
      @voter.vote.confirm!

      if @voter.election.embeddable? and params[:embed] == "true" \
        and @voter.election.early_results?
        redirect_to :action => :results, :id => @password, :embed => 'true'
      else
        render :action => 'thanks'
      end
    else
      redirect_to :action => 'index'
    end
  end
  
  def reminder
    if params[:email]
      voter_array= FullVoter.find(:all, :conditions => ["email = ?", params[:email]])
      voter_array.delete_if {|voter| voter.election.active == 0}
      unless voter_array.empty?
        VoterNotify.deliver_reminder(voter_array)
      end
      render :action => 'reminder_sent'
    end
  end
  
  def results
    if authenticate and
      (@voter.election.early_results? \
       or @voter.election.enddate < Time.now)
      
      @election = @voter.election
      # compute and display results

      @results = @election.results
      @candidates = {}
      @election.candidates.each {|c| @candidates[c.id] = c}
      @names = @election.names_by_id
        
      @sidebar_content = render_to_string(:partial => 'results_sidebar')
      #look for custom theme, and assign to instance variabels for widget use
      if @election.embed_custom_string
        @top_bar = SkinPicture.find(:first,
        :conditions => ["filename = ?", @election.embed_custom_string + "top_bar.png"])
        @default_image = SkinPicture.find(:first,
        :conditions => ["filename = ?", @election.embed_custom_string + "default_image.png"])
        @bg1 = SkinPicture.find(:first,
        :conditions => ["filename = ?", @election.embed_custom_string + "bg1.png"])
        @bg2 = SkinPicture.find(:first,
        :conditions => ["filename = ?", @election.embed_custom_string + "bg2.png"])
        @bottom_bar = SkinPicture.find(:first,
        :conditions => ["filename = ?", @election.embed_custom_string + "bottom_bar.png"])
      end
      if @election.embeddable? and params[:embed] == "true"
        render :template => 'embed/results', :layout => 'embed'
      else
        render :action => 'results'
      end
    else
      redirect_to :action => 'index'
    end
  end
  
  private
  def authenticate
    password = params[:id]
    if password == "open"
      election = Election.find(params[:format])
      unless election.authenticated?
        @voter = Voter.find(:all,
          :conditions => ["session_id = ? and election_id = ?",
                          session.session_id, election.id])[0]
        @password = "open." + election.id.to_s
      end
    else
      @voter = FullVoter.find(:all,
        :conditions => [ "password = ?", password ] )[0]
      @password = @voter.password
    end
    @voter
  end
end

